#!/usr/bin/env python3

from __future__ import absolute_import, division, print_function
from time import process_time
import energyflow as ef
import numpy as np
import matplotlib.pyplot as plt


class MassDistribution:
    def __init__(self, mass_list, event_list):
        self.event_list = event_list
        self.mass_list = mass_list

    def divide_mass_bins(self, mass_bin_size, cutoff_jets):
        mass_ranges = []
        max_mass = max(self.mass_list)
        min_mass = min(self.mass_list)
        previous_mass = min_mass
        for current_mass in np.arange(min_mass + mass_bin_size, max_mass, mass_bin_size):
            mass_ranges.append([previous_mass, current_mass])
            previous_mass = current_mass

        self.mass_ranges = mass_ranges

        #Sort indices into different mass ranges
        
        index_bins = []
        for mass_range in mass_ranges:
            index_bins.append([])

        index = 0
        for event_mass in self.mass_list:
            mass_bin = 0
            for mass_range in mass_ranges:
                if event_mass > mass_range[0] and event_mass <= mass_range[1]:
                    index_bins[mass_bin].append(index)
                    break
                mass_bin += 1
            index += 1

        #Remove bins that don't have enough jets   
        remove_indices = []
        i = 0
        for mass_bin in index_bins:
            if len(mass_bin) < cutoff_jets:
                remove_indices.append(i)
            i += 1
        
        for index in sorted(remove_indices, reverse=True):
            del index_bins[index]
            
        for index in sorted(remove_indices, reverse=True):
            del self.mass_ranges[index]

            
        #Create bins with event mass and particles
            
        self.event_mass_bins = []
        i = 0
        for mass_bin in index_bins:
            self.event_mass_bins.append([])
            for index in mass_bin:
                self.event_mass_bins[i].append(self.event_list[index])
            i += 1


    def get_mass_ranges(self):
        return self.mass_ranges

    def extract_jets_into_mass_bins(self):
        self.jet_mass_bins = []

        i = 0
        for mass_bin in self.event_mass_bins:
            self.jet_mass_bins.append([])
            for event in mass_bin:
                for jet in event:
                    self.jet_mass_bins[i].append(jet)
            i += 1

    def max_particles_per_jet(self):
        max_particles_per_jet = []
        for jet_mass_bin in self.jet_mass_bins:
            array_lengths = []
            for i in range(len(jet_mass_bin1)):
                array_lengths.append(len(jet_mass_bin1[i]))
            max_particles_per_jet.append(max(array_lengths))

        return max_particles_per_jet

    def pad_jet_arrays(self, num_particles):
        self.padded_jet_arrays = []
        
        for mass_bin in self.jet_mass_bins:
            jet_array = np.zeros((len(mass_bin),num_particles,6))
            for i in range(len(mass_bin)):
                for j in range(num_particles):
                    for k in range(6):
                        try:
                            jet_array[i,j,k] = mass_bin[i][j][k]
                        except IndexError:
                            jet_array[i,j,k] = 0
            self.padded_jet_arrays.append(jet_array)
            



#!/usr/bin/env python3

from __future__ import absolute_import, division, print_function
from time import process_time
import energyflow as ef
import numpy as np
import matplotlib.pyplot as plt
import MassDistribution

class SignalInjectionTwoGluons:
    def __init__(self, mass_list, event_jet_labels, event_list, mass_bin_size):
        self.mass_list = mass_list
        self.event_jet_labels = event_jet_labels
        self.event_list = event_list
        self.mass_bin_size = mass_bin_size

    def filter_gluon_events(self):
        indexes_gluon_events = []

        index = 0
        for event in self.event_jet_labels:
            if event[0] == 21 and event[1] == 21:
                indexes_gluon_events.append(index)
            index += 1

        self.mass_list_2_gluons = [self.mass_list[j] for j in indexes_gluon_events]
        self.event_list_2_gluons = [self.event_list[j] for j in indexes_gluon_events]
        

    def sort_gluon_events(self):
        gluon_events = new MassDistribution(self.mass_list_2_gluons, self.event_list_2_gluons)
        gluon_events.divide_mass_bins(self.mass_bin_size, 0)
        self.event_mass_2_gluons_bins = gluon_events.event_mass_bins

    def get_gluon_events_in_bin(self, index):
        return self.event_mass_2_gluons_bins[index]

    

        
        
                



#!/usr/bin/env python3

# standard library imports
from __future__ import absolute_import, division, print_function

# standard numerical library imports
import numpy as np

# energyflow imports
import energyflow as ef
from energyflow.archs import PFN
from energyflow.datasets import qg_jets
from energyflow.utils import data_split, remap_pids, to_categorical

# import libraries to supress output
from contextlib import contextmanager
import sys, os

# attempt to import sklearn
try:
    from sklearn.metrics import roc_auc_score, roc_curve
except:
    print('please install scikit-learn in order to make ROC curves')
    roc_curve = False

# attempt to import matplotlib
try:
    import matplotlib.pyplot as plt
except:
    print('please install matplotlib in order to make plots')
    plt = False
    
@contextmanager
def suppress_stdout():
    with open(os.devnull, "w") as devnull:
        old_stdout = sys.stdout
        sys.stdout = devnull
        try:  
            yield
        finally:
            sys.stdout = old_stdout
            
def PFN_AUC_calculation(jet_array_1, jet_array_2, train_size, test_size):
    X = np.concatenate([jet_array_1, jet_array_2])[:,:,:4]
    y = np.concatenate([np.ones(len(jet_array_1)), np.zeros(len(jet_array_2))])

    ################################### SETTINGS ###################################

    # data controls
    train, val, test = train_size, X.shape[0]-train_size-test_size, test_size
    use_pids = True

    # network architecture parameters
    Phi_sizes, F_sizes = (100, 100, 128), (100, 100, 100)

    # network training parameters
    num_epoch = 10
    batch_size = 500

    ################################################################################

    # convert labels to categorical
    Y = to_categorical(y, num_classes=2)

    # preprocess by centering jets and normalizing pts
    for x in X:
        mask = x[:,0] > 0
        yphi_avg = np.average(x[mask,1:3], weights=x[mask,0], axis=0)
        x[mask,1:3] -= yphi_avg
        x[mask,0] /= x[:,0].sum()

    # handle particle id channel
    if use_pids:
        remap_pids(X, pid_i=3)
    else:
        X = X[:,:,:3]

    # do train/val/test split 
    (X_train, X_val, X_test,
     Y_train, Y_val, Y_test) = data_split(X, Y, val=val, test=test)

    # build architecture
    pfn = 0
    with suppress_stdout():
        pfn = PFN(input_dim=X.shape[-1], Phi_sizes=Phi_sizes, F_sizes=F_sizes)

    # train model
    pfn.fit(X_train, Y_train,
              epochs=num_epoch,
              batch_size=batch_size,
              validation_data=(X_val, Y_val),
              verbose=0)

    # get predictions on test data
    preds = pfn.predict(X_test, batch_size=1000)

    # get area under the ROC curve
    auc = roc_auc_score(Y_test[:,1], preds[:,1])
    
    return auc


class PFNDist:
    def __init__(self, padded_jet_arrays):
        self.padded_jet_arrays = padded_jet_arrays

    def generate_AUCs(self, train_size, test_size):
        AUC_scores = []
        for i in range(len(self.padded_jet_arrays) - 1):
            auc = PFN_AUC_calculation(self.padded_jet_arrays[i], self.padded_jet_arrays[i+1],
                                      train_size, test_size)
            AUC_scores.append(auc)

        print(AUC_scores)
        
        return AUC_scores
